﻿using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Animation;

namespace Wp7nl.Behaviors
{
  /// <summary>
  /// A base class for behaviors doing 'something' with scaling
  /// </summary>
  public abstract class BaseScaleBehavior : SafeBehavior<FrameworkElement>
  {
    protected override void OnSetup()
    {
      base.OnSetup();
      SetRenderTransForm();
      AssociatedObject.RenderTransform = BuildTransform();
    }

    private void Activate()
    {
      if (AssociatedObject != null && AssociatedObject.RenderTransform is CompositeTransform)
      {
        var storyboard = BuildStoryBoard();
        if (storyboard != null)
        {
          storyboard.Begin();
        }
      }
    }

    private void SetRenderTransForm()
    {
      if (AssociatedObject != null)
      {
        AssociatedObject.RenderTransformOrigin = new Point(RenderTransformX, RenderTransformY);
      }
    }

    protected abstract CompositeTransform BuildTransform();

    protected abstract Storyboard BuildStoryBoard();

    #region Direction

    /// <summary>
    /// Direction Property name
    /// </summary>
    public const string DirectionPropertyName = "Direction";

    public Direction Direction
    {
      get { return (Direction)GetValue(DirectionProperty); }
      set { SetValue(DirectionProperty, value); }
    }

    /// <summary>
    /// Direction Property definition
    /// </summary>
    public static readonly DependencyProperty DirectionProperty = DependencyProperty.Register(
        DirectionPropertyName,
        typeof(Direction),
        typeof(BaseScaleBehavior),
        new PropertyMetadata(default(Direction)));

    #endregion
 
    #region Duration

    /// <summary>
    /// Duration Property name
    /// </summary>
    public const string DurationPropertyName = "Duration";

    public double Duration
    {
      get { return (double)GetValue(DurationProperty); }
      set { SetValue(DurationProperty, value); }
    }

    /// <summary>
    /// Duration Property definition
    /// </summary>
    public static readonly DependencyProperty DurationProperty = DependencyProperty.Register(
        DurationPropertyName,
        typeof(double),
        typeof(BaseScaleBehavior),
        new PropertyMetadata(500.0));

    #endregion

    #region Activated

    /// <summary>
    /// Activated Property name
    /// </summary>
    public const string ActivatedPropertyName = "Activated";

    public bool Activated
    {
      get { return (bool)GetValue(ActivatedProperty); }
      set { SetValue(ActivatedProperty, value); }
    }

    /// <summary>
    /// Activated Property definition
    /// </summary>
    public static readonly DependencyProperty ActivatedProperty = DependencyProperty.Register(
        ActivatedPropertyName,
        typeof(bool),
        typeof(BaseScaleBehavior),
        new PropertyMetadata(default(bool), ActivatedChanged));

    /// <summary>
    /// Activated property changed callback.
    /// </summary>
    /// <param name="d">The depency object (i.e. the behavior).</param>
    /// <param name="e">The property event args <see cref="System.Windows.DependencyPropertyChangedEventArgs"/> instance containing the event data.</param>  
    public static void ActivatedChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
    {
      var thisobj = d as BaseScaleBehavior;
      if (thisobj != null && e.OldValue != e.NewValue)
      {
        thisobj.Activate();
      }
    }

    #endregion

    #region RenderTransformX

    /// <summary>
    /// RenderTransformX Property name
    /// </summary>
    public const string RenderTransformXPropertyName = "RenderTransformX";

    public double RenderTransformX
    {
      get { return (double)GetValue(RenderTransformXProperty); }
      set { SetValue(RenderTransformXProperty, value); }
    }

    /// <summary>
    /// RenderTransformX Property definition
    /// </summary>
    public static readonly DependencyProperty RenderTransformXProperty = DependencyProperty.Register(
        RenderTransformXPropertyName,
        typeof(double),
        typeof(BaseScaleBehavior),
        new PropertyMetadata(0.5, RenderTransformChanged));
    #endregion

    #region RenderTransformY

    /// <summary>
    /// RenderTransformY Property name
    /// </summary>
    public const string RenderTransformYPropertyName = "RenderTransformY";

    public double RenderTransformY
    {
      get { return (double)GetValue(RenderTransformYProperty); }
      set { SetValue(RenderTransformYProperty, value); }
    }

    /// <summary>
    /// RenderTransformY Property definition
    /// </summary>
    public static readonly DependencyProperty RenderTransformYProperty = DependencyProperty.Register(
        RenderTransformYPropertyName,
        typeof(double),
        typeof(BaseScaleBehavior),
        new PropertyMetadata(0.5, RenderTransformChanged));

    #endregion

    /// <summary>
    /// RenderTransform X/Y property changed callback.
    /// </summary>
    /// <param name="d">The depency object (i.e. the behavior).</param>
    /// <param name="e">The property event args <see cref="System.Windows.DependencyPropertyChangedEventArgs"/> instance containing the event data.</param>  
    public static void RenderTransformChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
    {
      var thisobj = d as BaseScaleBehavior;
      if (thisobj != null)
      {
        thisobj.SetRenderTransForm();
      }
    }
  }
}
